using System;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace estoque
{
    /// <summary>
    /// Stellt die komplette Logging-Funktionalitt zur Verfgung
    /// </summary>
    public sealed class Tracer : INotifyPropertyChanged
    {
        private static Tracer tracer;

        public static Tracer Trace
        {
            get
            {
                if (Tracer.tracer == null)
                {
                    Tracer.tracer = new Tracer();
                }
                return Tracer.tracer;
            }
        }

        private ObservableCollection<Message> messages = new ObservableCollection<Message>();

        public ObservableCollection<Message> Messages
        {
            get { return messages; }
        }

        private TracerStatus currentDatabaseStatus;
        private TracerStatus currentSoftwareStatus;
        private TracerStatus currentStatus;

        /// <summary>
        /// Get or Set Gesamtstatus (Set wirft einen Changed-Event)
        /// </summary>
        public TracerStatus CurrentStatus
        {
            get
            {
                return this.currentStatus;
            }
            set
            {
                this.currentStatus = DecideCategory(this.currentStatus, value);
                this.InvokePropertyChanged("CurrentStatus");
            }
        }

        /// <summary>
        /// Get or Set DatenbankStatus (Set setzt auch CurrentStatus um)
        /// </summary>
        public TracerStatus CurrentDatabaseStatus
        {
            get { return this.currentDatabaseStatus; }
            set
            {
                this.currentDatabaseStatus = value;
                this.InvokePropertyChanged("CurrentDatabaseStatus");
                this.CurrentStatus = value;
            }
        }

        /// <summary>
        /// Get or Set SoftwareStatus (Set setzt auch CurrentStatus um)
        /// </summary>
        public TracerStatus CurrentSoftwareStatus
        {
            get { return this.currentSoftwareStatus; }
            set
            {
                this.currentSoftwareStatus = value;
                this.InvokePropertyChanged("CurrentSoftwareStatus");
                this.CurrentStatus = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Tracer"/> class.
        /// </summary>
        /// <remarks>CA1053</remarks>
        private Tracer() { }

        #region AddDatabase

        /// <summary>
        /// Fgt einen Eintrag ber die Datenbank zum Tracer hinzu
        /// </summary>
        private static void AddDatabase(TracerStatus status, string message, params object[] args)
        {
            Tracer.Trace.Add(message, status);
            Tracer.Trace.CurrentDatabaseStatus = DecideCategory(Tracer.Trace.CurrentDatabaseStatus, status);
        }

        /// <summary>
        /// Fgt einen Fehler ber die Datenbank zum Tracer hinzu
        /// </summary>
        public static void AddDatabaseError(string message, params object[] args)
        {
            Tracer.AddDatabase(TracerStatus.Error, message, args);
        }

        /// <summary>
        /// Fgt eine Warnung ber die Datenbank zum Tracer hinzu
        /// </summary>
        public static void AddDatabaseWarning(string message, params object[] args)
        {
            AddDatabase(TracerStatus.Warning, message, args);
        }

        /// <summary>
        /// Fgt eine Information ber die Datenbank zum Tracer hinzu
        /// </summary>
        public static void AddDatabaseInformation(string message, params object[] args)
        {
            AddDatabase(TracerStatus.Information, message, args);
        }

        #endregion AddDatabase

        #region AddSoftware

        /// <summary>
        /// Fgt einen Eintrag ber die Software zum Tracer hinzu
        /// </summary>
        private void AddSoftware(string message, TracerStatus status)
        {
            this.Add(message, status);
            this.CurrentSoftwareStatus = DecideCategory(Tracer.Trace.CurrentSoftwareStatus, status);
        }

        /// <summary>
        /// Fgt einen Fehler ber die Software zum Tracer hinzu
        /// </summary>
        public static void AddSoftwareError(string message)
        {
            Trace.AddSoftware(message, TracerStatus.Error);
        }

        /// <summary>
        /// Fgt eine Warnung ber die Software zum Tracer hinzu
        /// </summary>
        public static void AddSoftwareWarning(string message)
        {
            Trace.AddSoftware(message, TracerStatus.Warning);
        }

        /// <summary>
        /// Fgt eine Information ber die Software zum Tracer hinzu
        /// </summary>
        public static void AddSoftwareInformation(string message)
        {
            Trace.AddSoftware(message, TracerStatus.Information);
        }

        #endregion AddSoftware

        private void Add(string message, TracerStatus status)
        {
            messages.Add(new Message { Text = message, LogTime = DateTime.Now, Status = status });
        }

        /// <summary>
        /// Setzt alle Werte zurck und lscht alle Logging-Eintrge
        /// </summary>
        public static void ResetAll()
        {
            // Wir setzten die Stati zurck
            Tracer.Trace.Messages.Clear();
            Tracer.Trace.CurrentSoftwareStatus = TracerStatus.Information;
            Tracer.Trace.CurrentDatabaseStatus = TracerStatus.Information;
            Tracer.Trace.CurrentStatus = TracerStatus.Information;
        }

        private static TracerStatus DecideCategory(TracerStatus currentStatus, TracerStatus nextStatus)
        {
            if (nextStatus == TracerStatus.Error || currentStatus == TracerStatus.Error)
            {
                return TracerStatus.Error;
            }

            if (nextStatus == TracerStatus.Warning || currentStatus == TracerStatus.Warning)
            {
                return TracerStatus.Warning;
            }

            return TracerStatus.Information;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void InvokePropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler temp = this.PropertyChanged;
            if (temp != null)
            {
                temp(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}